
import math
import string

from vtool import qt
from vtool import qt_ui

from vtool import util

if util.is_in_maya():
    import maya.cmds as cmds
    
from vtool.maya_lib2 import rigs

class NodeWindow(qt_ui.BasicGraphicsWindow):
    def __init__(self):
        
        super(NodeWindow, self).__init__()
        self.setWindowTitle('Ramen')
        
    def _define_main_view(self):
        self.main_view = NodeView()
        
    def _node_connected(self, socket_item, node_item):
        
        exec_string = 'node_item._rig.%s = %s' % (socket_item.name, socket_item.value)
        exec(exec_string, {'node_item':node_item})
        
    def _node_disconnected(self, socket_item, node_item):
        
        if socket_item.name == 'joints':
            node_item._rig.joints = []
        
    def _node_selected(self, node_items):
        
        if node_items:
            self.side_menu.show()
            self.side_menu.nodes = node_items
        else:
            self.side_menu.hide()
            self.side_menu.nodes = []
            
    def _node_deleted(self, node_items):
        
        for node in node_items:
            if hasattr(node, '_rig'):
                node._rig.delete()
        
    def _build_widgets(self):
        
        self.main_view.main_scene.node_connect.connect(self._node_connected)
        self.main_view.main_scene.node_disconnect.connect(self._node_disconnected)
        self.main_view.main_scene.node_selected.connect(self._node_selected)
        self.main_view.main_scene.node_deleted.connect(self._node_deleted)
        
        self.side_menu = SideMenu()
        self.main_layout.addWidget(self.side_menu)    
        
        self.side_menu.hide()
        
class NodeView(qt_ui.BasicGraphicsView):
    
    def __init__(self, parent = None):
        super(NodeView, self).__init__(parent)
        
        self._zoom = 1
        self.drag = False
        
        self.setRenderHints(qt.QPainter.Antialiasing | qt.QPainter.HighQualityAntialiasing)
        
        brush = qt.QBrush()
        brush.setColor(qt.QColor(15,15,15,1))
        self.setBackgroundBrush(brush)
        #self.setRenderHints(qt.QPainter.Antialiasing | qt.QPainter.SmoothPixmapTransform | qt.QPainter.HighQualityAntialiasing)
        
        self.main_scene.addItem(NodeSocket())

    def drawBackground(self, painter, rect):
        
        pixmap = qt.QPixmap(40, 40)
        pixmap.fill(qt.QtCore.Qt.transparent)
        
        pix_painter = qt.QPainter(pixmap)
        pix_painter.setBrush(qt.QColor.fromRgbF(.15, .15, .15, 1))
        
        pen = qt.QPen()
        pen.setStyle(qt.Qt.NoPen)
        pix_painter.setPen(pen)
        pix_painter.drawRect(0,0,40,40)
        
        
        pen = qt.QPen()
        pen.setColor(qt.QColor.fromRgbF(0, 0, 0, .25))
        pen.setStyle(qt.Qt.SolidLine)
        
        pix_painter.setPen(pen)
        
        pix_painter.drawLine(0,0,2,0)
        pix_painter.drawLine(0,0,0,2)
        pix_painter.drawLine(38,0,40,0)
        pix_painter.drawLine(0,38,0,40)
        
        pix_painter.end()
        
        painter.fillRect(rect, pixmap)      

    def _define_main_scene(self):
        self.main_scene = NodeScene()
        
        self.main_scene.setObjectName('main_scene')
        self.main_scene.setSceneRect(0,0,5000,5000)
        
        self.setScene(self.main_scene)   

    def keyPressEvent(self, event):
        
        items = self.main_scene.selectedItems()
        
        if event.key() == qt.Qt.Key_F:
            
            
            position = items[0].pos()
            #position = self.mapToScene(items[0].pos())
            self.centerOn(position)
            
        if event.key() == qt.Qt.Key_Delete:
            for item in items:
                item.delete()
                
            
        super(NodeView, self).keyPressEvent(event)

    def wheelEvent(self, event):
        """
        Zooms the QGraphicsView in/out.
        
        """
        
        if self._zoom < 0.5 or self._zoom > 2:
            return
        
        inFactor = 1.25
        outFactor = 1 / inFactor
        oldPos = self.mapToScene(event.pos())
        if event.delta() > 0:
            zoomFactor = inFactor
        else:
            zoomFactor = outFactor
        
        self._zoom *= zoomFactor
        
        if self._zoom < 0.5:
            self._zoom = 0.5
            return
        if self._zoom > 2.0:
            self._zoom = 2.0
            return
        
        
        
        self.scale(zoomFactor, zoomFactor)
        newPos = self.mapToScene(event.pos())
        delta = newPos - oldPos
        self.translate(delta.x(), delta.y())

    def mousePressEvent(self, event):
        if event.button() == qt.QtCore.Qt.MiddleButton:
            self.setDragMode(qt.QGraphicsView.NoDrag)
            self.drag = True
            self.prevPos = event.pos()
            self.setCursor(qt.QtCore.Qt.SizeAllCursor)
        elif event.button() == qt.QtCore.Qt.LeftButton:
            
            self.setDragMode(qt.QGraphicsView.RubberBandDrag)
            #self.setDragMode(qt.QGraphicsView.RubberBandDrag)
            super(NodeView, self).mousePressEvent(event)
        

    def mouseMoveEvent(self, event):
        
        if self.drag:
            
            delta = (event.pos() - self.prevPos) * -1.0
            
            center = qt.QtCore.QPoint(self.viewport().width()/2.0 + delta.x(), self.viewport().height()/2.0 + delta.y())
            
            newCenter = self.mapToScene(center)
            
            self.centerOn(newCenter)
            self.prevPos = event.pos()
            return
        
        super(NodeView, self).mouseMoveEvent(event)

    def mouseReleaseEvent(self, event):
        if self.drag:
            self.drag = False
            self.setCursor(qt.QtCore.Qt.ArrowCursor)
            self.setDragMode(qt.QGraphicsView.RubberBandDrag)
        super(NodeView, self).mouseReleaseEvent(event) 
        

    def contextMenuEvent(self, event):
        super(NodeView, self).contextMenuEvent(event)
        
        if not event.isAccepted():
        
            self._build_context_menu(event)
        
        
    def _build_context_menu(self, event):
        
        self.menu = qt.QMenu()
        
        item_action_dict = {}
        
        self.menu.addSeparator()
        
        for node_name in register_item:
            
            item_action = qt.QAction(node_name, self.menu)
            self.menu.addAction(item_action)
            
            item_action_dict[item_action] = node_name
        
        self.menu.addSeparator()    
                
        action = self.menu.exec_(event.globalPos())
        
        pos = event.pos()
        pos = self.mapToScene(pos)
        
        if action in item_action_dict:
            node_name = item_action_dict[action]
            self.add_rig_item(node_name, pos)
            
    
    def add_rig_item(self, name, position):
        
        if name in register_item:
            
            item_inst = register_item[name](name)
            self.main_scene.addItem(item_inst)
            #item_inst.setPos(position)
            item_inst.setPos(position)    
        
        #box.setPos(window.main_view.main_scene.width()/2.0, window.main_view.main_scene.height()/2.0)
        
        
        
class NodeScene(qt.QGraphicsScene):    
    node_disconnect = qt.create_signal(object, object)
    node_connect = qt.create_signal(object, object) 
    node_selected = qt.create_signal(object) 
    node_deleted = qt.create_signal(object)
    
    def __init__(self):
        super(NodeScene, self).__init__()
        
        self.selectionChanged.connect(self._selection_changed)
        
    def _selection_changed(self):
        
        items = self.selectedItems()
        
        self.node_selected.emit(items)
    
        
class SideMenu(qt.QFrame):
    def __init__(self, parent = None):
        super(SideMenu, self).__init__(parent)
        self.setObjectName('side_menu')
        self._build_widgets()
        
        self._nodes = []
        self._group_widgets = []

    def _build_widgets(self):
        # Frame.
        self.setFixedWidth(200)
        
        
        self.main_layout = qt.QVBoxLayout(self)
        self.main_layout.setAlignment(qt.Qt.AlignCenter)
        
    def _clear_widgets(self):
        for widget in self._group_widgets:
                
            self.main_layout.removeWidget(widget)
            widget.deleteLater()
            
        self._group_widgets = []
        
    @property
    def nodes(self):
        return self._nodes
    
    @nodes.setter
    def nodes(self, nodes):
        self._nodes = nodes
        
        self._clear_widgets()
        
        for node in self._nodes:
            name = node.name
            
            group = qt_ui.Group(name)
            self.main_layout.addWidget(group)
            self._group_widgets.append(group)
            
            if hasattr(node, '_rig'):
                
                rig_class = node._rig
                node_attributes = rig_class._attribute.node
                
                for attribute in node_attributes:
                    
                    name, value, attr_type = attribute
                    
                    if attr_type == rigs.AttrType.STRING:
                        string_attr = qt_ui.GetString(name)
                        string_attr.set_text(value)
                        group.main_layout.addWidget(string_attr)
              
class ColorPickerItem(qt.QGraphicsObject):
    
    color_changed = qt_ui.create_signal(object)
    
    def __init__(self):
        super(ColorPickerItem, self).__init__()
        
        self.rect = qt.QtCore.QRect(10,10,50,20)
        
        
        # Brush.
        self.brush = qt.QBrush()
        self.brush.setStyle(qt.QtCore.Qt.SolidPattern)
        self.brush.setColor(qt.QColor(90,90,90,255))

        # Pen.
        self.pen = qt.QPen()
        self.pen.setStyle(qt.QtCore.Qt.SolidLine)
        self.pen.setWidth(1)
        self.pen.setColor(qt.QColor(20,20,20,255))

        self.selPen = qt.QPen()
        self.selPen.setStyle(qt.QtCore.Qt.SolidLine)
        self.selPen.setWidth(3)
        self.selPen.setColor(qt.QColor(255,255,255,255))

    def paint(self, painter, option, widget):
        painter.setBrush(self.brush)
        if self.isSelected():
            painter.setPen(self.selPen)
        else:
            painter.setPen(self.pen)
        
        painter.drawRoundedRect(self.rect, 10,10)
        
        #painter.drawRect(self.rect)

    def mousePressEvent(self, event):
        
        super(ColorPickerItem, self).mousePressEvent(event)
        
        color_dialog = qt.QColorDialog
        color = color_dialog.getColor()
        
        self.brush.setColor(color)
        self.update()
        
        self.color_changed.emit(color)

    def boundingRect(self):
        return qt.QtCore.QRectF(self.rect)
    
        
#--- sockets

class NodeSocket(qt.QGraphicsItem):

    def __init__(self, socket_type = 'in', name = None, value = None, data_type = None):
        
        self.name = name
        self.value = value
        self.data_type = data_type
        
        if self.name:
            split_name = self.name.split('_')
            if split_name:
                found = []
                for name in split_name:
                    name = name.capitalize()
                    found.append(name)
                self.nice_name = string.join(found)
            else:
                self.nice_name = self.name.capitalize()
        else:
            self.nice_name = None
        
        super(NodeSocket, self).__init__()
        
        self._socket_type = socket_type
        
        self.rect = qt.QtCore.QRectF(0,0,0,0)        
        
        self.side_socket_height = 12

        # Brush.
        self.brush = qt.QBrush()
        self.brush.setStyle(qt.QtCore.Qt.SolidPattern)
        self.brush.setColor(qt.QColor(60,60,60,255))
        

        # Pen.
        self.pen = qt.QPen()
        
        self.socket_color = qt.QColor(60,60,60,255)
        
        self.pen.setColor(qt.QColor(20,20,20,255))
        
        if data_type == rigs.AttrType.TRANSFORM:
            self.socket_color = qt.QColor(100,200,100,255)
        if data_type == rigs.AttrType.STRING:
            self.socket_color = qt.QColor(100,150,220,255)
        if data_type == rigs.AttrType.COLOR:
            self.socket_color = qt.QColor(220,150,100,255)
        
        self.brush.setColor(self.socket_color)
        
        if socket_type == 'in':
            
            self.rect = qt.QtCore.QRect(-10, self.side_socket_height, 20,20)
            
            #self.setFlag(self.ItemStacksBehindParent)
            
        if socket_type == 'out':
            
            self.rect = qt.QtCore.QRect(148, self.side_socket_height, 20,20)
            
        if socket_type == 'top':
            self.rect = qt.QtCore.QRect(10, -13, 15,15)
        
        


        # Lines.
        #self.out_lines = []
        #self.in_lines = []
        self.lines = []
    


    def boundingRect(self):
        return qt.QtCore.QRectF(self.rect)

    def paint(self, painter, option, widget):
        painter.setBrush(self.brush)
        painter.setPen(self.pen)
        self.pen.setStyle(qt.QtCore.Qt.NoPen)
        self.pen.setWidth(0)
        painter.setPen(self.pen)
        
        
        
        if self._socket_type == 'in':      
            
            rect = qt.QtCore.QRectF(self.rect)
            rect.adjust(3,3,-3,-3)
            painter.drawEllipse(rect)
            
            self.pen.setStyle(qt.QtCore.Qt.SolidLine)
            self.pen.setWidth(1) 
            painter.setPen(self.pen)
            
            painter.drawText(qt.QtCore.QPoint(15,self.side_socket_height+12),self.nice_name)
            
        if self._socket_type == 'out':
            
            poly = qt.QPolygon()
            #poly << qt.QtCore.QPoint(0,0) << qt.QtCore.QPoint(0,16) << qt.QtCore.QPoint(20,10) << qt.QtCore.QPoint(20,6)
            poly.append( qt.QtCore.QPoint(0,2) )
            poly.append( qt.QtCore.QPoint(0,18) )
            poly.append( qt.QtCore.QPoint(6,18) )
            #poly.append( qt.QtCore.QPoint(6,13) )
            poly.append( qt.QtCore.QPoint(14,12) )
            poly.append( qt.QtCore.QPoint(15,10) )
            poly.append( qt.QtCore.QPoint(14,8) )
            #poly.append( qt.QtCore.QPoint(6, 7) )
            poly.append( qt.QtCore.QPoint(6, 2) )
            
            
            poly.translate(self.rect.x(), self.rect.y())
            painter.drawPolygon(poly)
            
            self.pen.setStyle(qt.QtCore.Qt.SolidLine)
            self.pen.setWidth(1)
            painter.setPen(self.pen)
            name_len = painter.fontMetrics().width(self.nice_name)
            offset = 140 - name_len
            painter.drawText(qt.QtCore.QPoint(offset,self.side_socket_height+12),self.nice_name)
            
        if self._socket_type == 'top':
            
            rect = qt.QtCore.QRectF(self.rect)
            painter.drawRect(rect)
             
            self.pen.setStyle(qt.QtCore.Qt.SolidLine)
            self.pen.setWidth(1)
            painter.setPen(self.pen)
            height = self.rect.height()
            
            painter.drawText(qt.QtCore.QPoint(15,height+5), self.nice_name)
            

    def mousePressEvent(self, event):
        
        self.new_line = None
        
        if self._socket_type == 'out':
            
            pointA = self.get_center()
            
            pointB = self.mapToScene(event.pos())
            self.new_line = NodeLine(pointA, pointB)
            
            #self.new_line.stackBefore()
            
        elif self._socket_type == 'in':
            
            pointA = self.mapToScene(event.pos())
            pointB = self.get_center()
                        
            self.new_line = NodeLine(pointA, pointB)
            
            #self.new_line.stackBefore()

        else:
            super(NodeSocket, self).mousePressEvent(event)        
        """    
        elif self._socket_type == 'top':
            pointA = self.mapToScene(event.pos())
            pointB self.get_center()
            
            self.new_line = NodeLine(pointA,pointB)
            self.lines.append(self.new_line)
            self.scene().addItem(self.new_line)
        """    
        
        self.lines.append(self.new_line)
        self.scene().addItem(self.new_line)
        
        if self.new_line:
            self.new_line.color = self.socket_color
            

    def mouseMoveEvent(self, event):
        if self._socket_type == 'out':
            pointB = self.mapToScene(event.pos())
            self.new_line.pointB = pointB
        elif self._socket_type == 'in':
            pointA = self.mapToScene(event.pos())
            self.new_line.pointA = pointA
        else:
            super(NodeSocket, self).mouseMoveEvent(event)

    def mouseReleaseEvent(self, event):
        
        self.new_line.hide()
        
        item = self.scene().itemAt(event.scenePos().toPoint(), qt.QTransform())
        
        self.new_line.show()
        
        if item:
            if self.data_type != item.data_type:
                self.remove_line(self.new_line)
                self.new_line = None
                util.warning('Cannot connect sockets of different type.')
                return
        if item == self.new_line or not hasattr(item, '_socket_type'):
            self.remove_line(self.new_line)
            self.new_line = None
            return
        if self._socket_type == 'out' and item._socket_type == 'in':
            self.new_line.source = self
            self.new_line.target = item
            item.lines.append(self.new_line)
            
            self.new_line.pointB = item.get_center()
            
            self.scene().node_connect.emit(self, item.parentItem())
            
        elif self._socket_type == 'out' and item._socket_type == 'top':
            self.new_line.source = self
            self.new_line.target = item
            item.lines.append(self.new_line)
            self.new_line.pointB = item.get_center()
            
            self.scene().node_connect.emit(self, item.parentItem())
            
            
        elif self._socket_type == 'in' and item._socket_type == 'out':
            self.new_line.source = item
            self.new_line.target = self
            item.lines.append(self.new_line)
            self.new_line.pointA = item.get_center()
            
            self.scene().node_connect.emit(item, self.parentItem())
        else:
            super(NodeSocket, self).mouseReleaseEvent(event)

    def remove_line(self, line_item):
        
        removed = False
        
        if line_item in self.lines:
            self.lines.remove(line_item)
            removed = True
        
        if removed:
            self.scene().removeItem(line_item)

    def get_center(self):
        rect = self.boundingRect()
        
        if self._socket_type == 'out':
            center = qt.QtCore.QPointF(rect.x() + rect.width()-5, rect.y() + rect.height()/2)      
        if self._socket_type == 'in':
            center = qt.QtCore.QPointF(rect.x() + rect.width()/2.0, rect.y() + rect.height()/2.0)
        if self._socket_type == 'top':
            center = qt.QtCore.QPointF(rect.x() + rect.width()/2.0, rect.y() + rect.height()/2.0)
        
        center = self.mapToScene(center)
        
        return center   
        
    
class NodeLine(qt.QGraphicsPathItem):
    
    def __init__(self, pointA, pointB):
        super(NodeLine, self).__init__()
        self._pointA = pointA
        self._pointB = pointB
        self._source = None
        self._target = None
        self.setZValue(3)
        
        self.brush = qt.QBrush()
        self.brush.setStyle(qt.QtCore.Qt.SolidPattern)
        self.brush.setColor(qt.QColor(200,200,200,255))
        
        
        self.pen = qt.QPen()
        self.pen.setStyle(qt.QtCore.Qt.SolidLine)
        self.pen.setWidth(1)
        self.pen.setColor(qt.QColor(200,200,200,255))
        self.setPen(self.pen)

    def mousePressEvent(self, event):
        self.pointB = event.pos()

    def mouseMoveEvent(self, event):
        self.pointB = event.pos()
        
    def mouseReleaseEvent(self, event):
        
        self._target.remove_line(self)
        
        self._target.scene().node_disconnect.emit(self._target, self._target.parentItem())

    def update_path(self):
        path = qt.QPainterPath()
        path.moveTo(self.pointA)
        dx = self.pointB.x() - self.pointA.x()
        dy = self.pointB.y() - self.pointA.y()
        #ctrl1 = qt.QtCore.QPointF(self.pointA.x() + dx * 0.25, self.pointA.y() + dy * 0.1)
        #ctrl2 = qt.QtCore.QPointF(self.pointA.x() + dx * 0.75, self.pointA.y() + dy * 0.9)
        ctrl1 = qt.QtCore.QPointF(self.pointA.x() + dx * 0.5, self.pointA.y() + dy * 0.1)
        ctrl2 = qt.QtCore.QPointF(self.pointA.x() + dx * 0.5, self.pointA.y() + dy * 0.9)
        
        path.cubicTo(ctrl1, ctrl2, self.pointB)
        
        self.setPath(path)

    def paint(self, painter, option, widget):
        
        if hasattr(self, 'color'):
            color = self.color.darker(70)
            self.brush.setColor(color)
            self.pen.setColor(color)
        
        path = self.path()
        painter.setPen(self.pen)
        painter.drawPath(path)
        
        painter.setBrush(self.brush)
        painter.drawEllipse(self.pointB.x() - 2,self.pointB.y() - 2 ,5,5)
        
        #draw arrow
        
        if path.length() < 50:
            return
        
        point = path.pointAtPercent(0.5)
        point_test = path.pointAtPercent(0.51)
        
        point_orig = qt.QtCore.QPointF(point.x() + 1.0, point.y())
        
        point_orig = point_orig-point
        point_test = point_test-point
        
        dot = point_orig.x()*point_test.x() + point_orig.y()*point_test.y()
        det = point_orig.x()*point_test.y() - point_orig.y()*point_test.x()
        angle = math.atan2(det, dot)
        
        poly = qt.QPolygonF()
        poly.append( qt.QtCore.QPointF(math.cos(angle) * 0 - math.sin(angle) * -5,  
                                       math.sin(angle) * 0 + math.cos(angle) * -5) )
        poly.append( qt.QtCore.QPointF(math.cos(angle) * 10 - math.sin(angle) * 0,  
                                 math.sin(angle) * 10 + math.cos(angle) * 0)   )
         
        poly.append( qt.QtCore.QPointF(math.cos(angle) * 0 - math.sin(angle) * 5,
                                 math.sin(angle) * 0 + math.cos(angle) * 5)     )
        
        poly.translate(point.x(), point.y())
        
        painter.drawPolygon(poly)
        
    @property
    def pointA(self):
        return self._pointA

    @pointA.setter
    def pointA(self, point):
        self._pointA = point
        self.update_path()

    @property
    def pointB(self):
        return self._pointB

    @pointB.setter
    def pointB(self, point):
        self._pointB = point
        self.update_path()

    @property
    def source(self):
        return self._source

    @source.setter
    def source(self, widget):
        self._source = widget

    @property
    def target(self):
        return self._target

    @target.setter
    def target(self, widget):
        self._target = widget

#--- nodes

class NodeItem(qt.QGraphicsItem):
    def __init__(self, name = ''):
        self._current_socket_pos = 0
        
        super(NodeItem, self).__init__()
        self.name = name
        
        self.rect = qt.QtCore.QRect(0,0,150,40)
        self.setFlag(qt.QGraphicsItem.ItemIsMovable)
        self.setFlag(qt.QGraphicsItem.ItemIsSelectable)
        
        
        # Brush.
        self.brush = qt.QBrush()
        self.brush.setStyle(qt.QtCore.Qt.SolidPattern)
        self.brush.setColor(qt.QColor(90,90,90,255))

        # Pen.
        self.pen = qt.QPen()
        self.pen.setStyle(qt.QtCore.Qt.SolidLine)
        self.pen.setWidth(2)
        self.pen.setColor(qt.QColor(120,120,120,255))

        self.selPen = qt.QPen()
        self.selPen.setStyle(qt.QtCore.Qt.SolidLine)
        self.selPen.setWidth(3)
        self.selPen.setColor(qt.QColor(255,255,255,255))
        
        self._in_sockets = []
        self._out_sockets = []
        self._build_items()
        
    """
    def shape(self):
        path = qt.QPainterPath()
        path.addRect(self.boundingRect())
        return path
    """
    
    def _build_items(self):
        #self.add_socket(socket_type, data_type, name)
        pass
        
    def boundingRect(self):
        return qt.QtCore.QRectF(self.rect)

    def paint(self, painter, option, widget):
        painter.setBrush(self.brush)
        if self.isSelected():
            painter.setPen(self.selPen)
        else:
            painter.setPen(self.pen)
        
        painter.drawRoundedRect(self.rect, 10,10)
        
        pen = qt.QPen()
        pen.setStyle(qt.QtCore.Qt.SolidLine)
        pen.setWidth(1)
        pen.setColor(qt.QColor(255,255,255,255))   
        painter.setPen(pen)     
        painter.drawText(35,-10,self.name )
        
        self.setZValue(1)
        #painter.drawRect(self.rect)

    def mouseMoveEvent(self, event):
        super(NodeItem, self).mouseMoveEvent(event)
        
        for socket in self._out_sockets:
            for line in socket.lines:
                line.pointA = line.source.get_center()
                line.pointB = line.target.get_center()
        
        for socket in self._in_sockets:
            for line in socket.lines:
                line.pointA = line.source.get_center()
                line.pointB = line.target.get_center()

    #def mousePressEvent(self, event):
    #    super(NodeItem, self).mousePressEvent(event)
        
    #    print 'here!!!'
    #    self.scene().node_selected.emit(self)

    def contextMenuEvent(self, event):
        self._build_context_menu(event)
        event.setAccepted(True)
        
    def _build_context_menu(self, event):
        
        menu = qt.QMenu()
        
        add_in_socket = menu.addAction('add in socket')
        add_out_socket = menu.addAction('add out socket')
        add_top_socket = menu.addAction('add top socket')
        
        selected_action = menu.exec_(event.screenPos())

        if selected_action == add_top_socket:
            self.add_top_socket('parent','', None)

        if selected_action == add_in_socket:
            self.add_in_socket('goo','', None)
            
        if selected_action == add_out_socket:
            self.add_out_socket('foo','', None)
         
    def _add_space(self, item):
        
        if self._current_socket_pos > 0:
            height = self.rect.height()
            height += 20
        
            self.rect = qt.QtCore.QRect(0,0,150,height)
        
            item.setY(self._current_socket_pos*20)
            
        self._current_socket_pos += 1

    def add_top_socket(self, name, value, data_type):
        
        socket = NodeSocket('top', name, value, data_type)
        socket.setParentItem(self)
        
        self._in_sockets.append(socket)
        #self._add_space(socket)
    
    def add_in_socket(self, name, value, data_type):
        
        socket = NodeSocket('in', name, value, data_type)
        socket.setParentItem(self)
        
        self._in_sockets.append(socket)
        self._add_space(socket)
    
    def add_out_socket(self, name, value, data_type):
        
        socket = NodeSocket('out', name, value, data_type)
        socket.setParentItem(self)
        
        self._out_sockets.append(socket)
        self._add_space(socket)
        
        
    def add_color_picker(self):
        
        color_picker = ColorPickerItem()
        color_picker.setParentItem(self)
        
        return color_picker
        
        
    def add_line_edit(self):
        
        text_entry = qt.QLineEdit()
        text_entry.setMaximumWidth(130)
        text_entry.setMaximumHeight(20)
        text_entry.setPlaceholderText('joint search')
        #text_entry.setEchoMode(text_entry.NoEcho)
        proxy_widget = qt.QGraphicsProxyWidget(self)
        proxy_widget.setWidget(text_entry)
        proxy_widget.setPos(10,10)
        
        
        self._add_space(proxy_widget)
        
        return text_entry
        
    def delete(self):
        
        if not self.scene():
            return

        other_sockets = {}
        
        for socket in self._in_sockets:
            
            for line in socket.lines:
                line.target = None
                
                if not line.source in other_sockets:
                    other_sockets[line.source] = [] 
                    
                other_sockets[line.source].append(line)
                
                self.scene().removeItem(line)

            socket.lines = []
        
        for socket in self._out_sockets:
        
            for line in socket.lines:
                line.source = None
                
                if not line.target in other_sockets:
                    other_sockets[line.target] = []
                
                other_sockets[line.target].append(line)
                    
                self.scene().removeItem(line)
                    
        
            socket.lines = []
        
        for socket in other_sockets:
            lines = other_sockets[socket]
            
            for line in lines:
                if line in socket.lines:
                    socket.lines.remove(line)
            
        
        
        self.scene().removeItem(self)

        
        
        

    def get_socket(self, input_name):
        
        sockets = self._in_sockets + self._out_sockets
        
        for socket in sockets:
            if socket.name == input_name:
                return socket
    
    def get_input(self, name):
        found = []
        
        for socket in self._in_sockets:
            
            if socket.name == name:
                for line in socket.lines:
                    found.append(line.source)
                    
        return found        

    def get_outputs(self, name):
        
        found = []
        
        for socket in self._out_sockets:
            
            if socket.name == name:
                
                for line in socket.lines:
                    found.append(line.target)
                    
        return found
    
    def run(self):
          
        util.show('Run %s' % self.__class__.__name__)

class ColorItem(NodeItem):
    
    def _build_items(self):
        
        picker = self.add_color_picker()
        
        picker.color_changed.connect(self._color_changed) 
        
        self.add_out_socket('color', None, rigs.AttrType.COLOR)
        
    def _color_changed(self, color):
        color_value =  color.getRgbF()
        color_value = [color_value[0], color_value[1], color_value[2]]
        
        self.color = color_value
        
        
        
        
        self.run()
    
    def run(self):
        
        socket = self.get_socket('color')
        socket.value = self.color
        
        outputs = self.get_outputs('color')
        for output in outputs:
            
            node = output.parentItem()
            node.run(socket)
             


class JointsItem(NodeItem):
    def _build_items(self):
        
        text_entry = self.add_line_edit()
        self.add_out_socket('joints', [], rigs.AttrType.TRANSFORM)
        #self.add_socket(socket_type, data_type, name)
        
        self._joint_entry_widget = text_entry
        text_entry.returnPressed.connect(self.run)
        
    def _get_joints(self):
        filter_text = self._joint_entry_widget.text()
        
        joints = []
        if util.is_in_maya():
            joints = cmds.ls(filter_text, type = 'joint')
            
        return joints
        
    def run(self):
        super(JointsItem, self).run()
        
        joints = self._get_joints()
        
        socket = self.get_socket('joints')
        socket.value = joints
        
        outputs = self.get_outputs('joints')
        for output in outputs:
            node = output.parentItem()
            
            node.run(socket)
         
        return joints
        
        

class RigItem(NodeItem):
    
    def __init__(self, name = ''):
        super(RigItem, self).__init__(name)
    
        self._current_socket_pos = 1
    
        self._rig = self._init_rig_class_instance()
        
        if self._rig:
            ins = self._rig.get_in()
            outs = self._rig.get_out()
            
            for in_value in ins:
                name, value, attr_type = in_value 
                if name == 'parent':
                    self.add_top_socket(name, value, attr_type)
                else:
                    self.add_in_socket(name, value, attr_type)
            
            for out_value in outs:
                name, value, attr_type = out_value
                
                self.add_out_socket(name, value, attr_type)    
    
    def _init_rig_class_instance(self):
        return None

    def run(self, socket = None):
        super(RigItem, self).run()
        
        util.show('Running %s' % self._rig.__class__.__name__)
        
        exec_string = 'self._rig.%s = %s' % (socket.name, socket.value)
        exec(exec_string)
        
    def delete(self):
        super(RigItem, self).delete()
        
        self._rig.delete()
            

class FkItem(RigItem):
    
    def _init_rig_class_instance(self):
        return rigs.Fk()

    """
    def run(self):
        super(FkItem, self).run()
        
        joints_socket = self.get_input('joints')
        
        joints = []
        if joints_socket:
            joints = joints_socket[0].value
        
        self._rig.joints = joints
    """ 

class IkItem(RigItem):
    
    def _init_rig_class_instance(self):
        return rigs.Ik()

#--- registry

register_item = {
    'FkRig' : FkItem,
    'IkRig' : IkItem,
    'Joints': JointsItem,
    'Color' : ColorItem
}